@@ -9,18 +9,19 @@
    A non-zero exit code also indicates failure.
 */
 
+/* This should probably be moved to glib/testglib.c.  */
 int main(int argc, char *argv[])
 {
 	char **output;
 	int i;
-	gnomelib_init("split");
-	output = gnome_string_split("foo:bar:baz", ":", -1);
+	gnomelib_init("split", "0.0");
+	output = g_strsplit("foo:bar:baz:qaz", ":", -1);
 	for(i = 0; output[i]; i++) {
 		g_print("%s ", output[i]);
 		g_free(output[i]);
 	}
 	g_print("\n"); g_free(output);
-	output = gnome_string_split("foo:bar:baz", ":", 2);
+	output = g_strsplit("foo:bar:baz:qaz", ":", 2);
 	for(i = 0; output[i]; i++) {
 		g_print("%s ", output[i]);
 		g_free(output[i]);
